#ifndef __RT_TIME_H__
#define __RT_TIME_H__

/*
 *  linux/include/asm/rt_time.h
 *
 * low-level time-related routines
 *
 * Written by Michael Barabanov
 * Copyright VJY Associates
 * Released under the terms of the LGPL
 *
 */



#ifndef rtl_div_t
typedef struct {
	int quot;	/* Quotient.  */
	int rem;	/* Remainder.  */
} rtl_div_t;
#endif



/*  a * mult / div  */

static inline rtl_div_t imuldivmod(int a, int mult, int div)
{
	register rtl_div_t res;
	__asm__("imull %1 ; idivl %3"
	:	"=a"(res.quot),
		"=d"(res.rem)
	:	"1"(mult),
		"r"(div),
		"0"(a)
      	:	"dx");
	return res;
}

/* a * mult / div */

static inline int imuldiv(int a, int mult, int div)
{
	__asm__("imull %1 ; idivl %2"
		:"=a" (a)
		:"d" (mult),
		 "r" (div),
		 "0" (a)
		:"dx");
	return a;
}


#ifdef __KERNEL__

#include <linux/time.h>
#include <linux/delay.h>

/* shouldn't this function take RTIME values instead? */
static inline void rt_delay(long units)
{
	units *= 3599;		/* 2**32 / RT_TICKS_PER_SEC */
	__asm__("mull %0"
		:"=d" (units)
		:"a" (units),"0" (loops_per_sec)
		:"ax");
		
	__delay(units);
}

#endif


#define RT_TICKS_PER_SEC 1193180LL
#define RT_TIME_END 0x7fffFfffFfffFfffLL

/* for larger periods we will have problems with jiffies */
#define RT_MAX_PERIOD 10000

typedef long long RTIME;

extern RTIME rt_linux_clock;


extern RTIME (*do_rt_get_time)(void);

static inline RTIME rt_get_time(void)
{
	return do_rt_get_time();
}

extern int rt_request_timer(void (*fn)(void));
extern void rt_free_timer(void);
extern void rt_set_timer(RTIME when);
extern void rt_no_timer(void);

static inline RTIME rt_ts_to_rtime(struct timespec *res)
{
	return res->tv_sec * RT_TICKS_PER_SEC + imuldiv(res->tv_nsec, RT_TICKS_PER_SEC, 1000000000);
}

extern RTIME rt_get_prof_time(void);

extern void I8253_disable_speaker(void);
extern void I8253_enable_speaker(int );
extern void I8253_make_periodic(unsigned int period);
extern void I8253_make_oneshot(unsigned int);
#endif
